#pragma once

#include "Ray.h"

//Holds material information of a particular object
class Material
{
public:
	Material();
	//Material values used for lighting equations
	glm::vec3 ambient;
	glm::vec3 diffuse;
	glm::vec3 specular;	
	float specularExponent;
	//TODO add further material values here such as reflection/refraction index
};

//Interface for an objet in the scene
class Object
{
public:
	//Constructor
	//@transform The transformation matrix for the object
	//@material The material properties of the object
	Object(const glm::mat4 &transform = glm::mat4(1.0f), const Material &material = Material());

	//Test whether a ray intersects the object
	//@ray The ray that we are testing for intersection
	//@info Object containing information on the intersection between the ray and the object(if any)
	virtual bool Intersect(const Ray &ray, IntersectInfo &info) const { return true; }

	//Retrun the position of the object, according to its transformation matrix
	glm::vec3 Position() const { return glm::vec3(_transform[3][0],_transform[3][1],_transform[3][2]); }

	//Get a pointer to the objects material properties
	const Material *MaterialPtr() const { return &_material; }

protected:
	glm::mat4 _transform;
	Material _material;
};